#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>

/* Base-64 decoding.  This represents binary data as printable ASCII
** characters.  Three 8-bit binary bytes are turned into four 6-bit
** values, like so:
**
**   [11111111]  [22222222]  [33333333]
**
**   [111111] [112222] [222233] [333333]
**
** Then the 6-bit values are represented using the characters "A-Za-z0-9+/".
*/

static int b64_decode_table[256] = {
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 00-0F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 10-1F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,62,-1,-1,-1,63,  /* 20-2F */
    52,53,54,55,56,57,58,59,60,61,-1,-1,-1,-1,-1,-1,  /* 30-3F */
    -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,  /* 40-4F */
    15,16,17,18,19,20,21,22,23,24,25,-1,-1,-1,-1,-1,  /* 50-5F */
    -1,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,  /* 60-6F */
    41,42,43,44,45,46,47,48,49,50,51,-1,-1,-1,-1,-1,  /* 70-7F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 80-8F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 90-9F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* A0-AF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* B0-BF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* C0-CF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* D0-DF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* E0-EF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1   /* F0-FF */
    };

/* Do base-64 decoding on a string.  Ignore any non-base64 bytes.
** Return the actual number of bytes generated.  The decoded size will
** be at most 3/4 the size of the encoded, and may be smaller if there
** are padding characters (blanks, newlines).
*/
static int
b64_decode( const char* str, unsigned char* space, int size )
    {
    const char* cp;
    int space_idx, phase;
    int d, prev_d = 0;
    unsigned char c;

    space_idx = 0;
    phase = 0;
    for ( cp = str; *cp != '\0'; ++cp )
	{
	d = b64_decode_table[(int) *cp];
	if ( d != -1 )
	    {
	    switch ( phase )
		{
		case 0:
		++phase;
		break;
		case 1:
		c = ( ( prev_d << 2 ) | ( ( d & 0x30 ) >> 4 ) );
		if ( space_idx < size )
		    space[space_idx++] = c;
		++phase;
		break;
		case 2:
		c = ( ( ( prev_d & 0xf ) << 4 ) | ( ( d & 0x3c ) >> 2 ) );
		if ( space_idx < size )
		    space[space_idx++] = c;
		++phase;
		break;
		case 3:
		c = ( ( ( prev_d & 0x03 ) << 6 ) | d );
		if ( space_idx < size )
		    space[space_idx++] = c;
		phase = 0;
		break;
		}
	    prev_d = d;
	    }
	}
    return space_idx;
    }


int main (int argc, char **argv)
{
	// A VERY basic XML parser!

	if (argc < 2)
	{
	 	fprintf (stderr, "usage: get_pwd CONFIG_FILE\n");
		return 1;
	}

	FILE *fp = fopen (argv[1], "r");

	if (!fp)
	 fprintf (stderr, "cannot open file `%s'", argv[1]);

	int c;
	int inTag = 0;
	int inPassword = 0;
	int inUsername = 0;
	char tagName[400];
	char password[100];
	char username[100] = "admin";
	char passwordDecode[100];
	int l;

	int i = 0;
	while ((c = getc (fp)) != EOF)
	{
		if (c == '<')
		{
			if (inPassword)
			{
				i = 0;
				inPassword = 0;

				if (username[0] != 0)
				{
					l = b64_decode(&(password[0]), passwordDecode, sizeof(passwordDecode) - 1 );
					passwordDecode[l] = '\0';

					printf("%s:%s\n", username, passwordDecode);
					return;
				}
			}
			if (inUsername)
			{
				i = 0;
				inUsername = 0;

				if (password[0] != 0)
				{
					l = b64_decode(&password, (unsigned char*) passwordDecode, sizeof(passwordDecode) - 1 );
					passwordDecode[l] = '\0';

					printf("%s:%s\n", username, passwordDecode);
					return;
				}
			}

			// Hit start of tag
			inTag = 1;
		}
		else if (c == '>')
		{
			inTag = 0;
			i = 0;

			if (strcmp(tagName, "AdminPassword") == 0)
			{
				inPassword = 1;
			}
			else if (strcmp(tagName, "AdminUserName") == 0)
			{
				inUsername = 1;
			}
		}
		else if (inTag)
		{
			if (i < 400)
			{
				tagName[i] = c;
				tagName[i + 1] = 0;
				i++;
			}
			else
			{
				fprintf (stderr, "Tag name too big!\n");
				return 3;
			}
		}
		else if (inPassword)
		{
			if (i < 100)
			{
				password[i] = c;
				password[i + 1] = 0;
				i++;
			}
			else
			{
				fprintf (stderr, "Password too big!\n");
				return 3;
			}
		}
		else if (inUsername)
		{
			if (i < 100)
			{
				username[i] = c;
				username[i + 1] = 0;
				i++;
			}
			else
			{
				fprintf (stderr, "Username too big!\n");
				return 3;
			}
		}
	}

	fclose (fp);
	return 2;

}
